@@ -1,6 +1,14 @@
 #!/usr/bin/env python
-"""Simple program to exact 5x5 magnetic variations in WMM2015
-from MagneticField for use in geoid.c"""
+# This code run compatibly under Python 2 and 3.x for x >= 2.
+# Preserve this property!
+#
+# This file is Copyright 2019 by the GPSD project
+# SPDX-License-Identifier: BSD-2-Clause
+
+"""Simple program to extract WMM2015 data for use in geoid.c
+
+Extract 5x5 magnetic variations in WMM2015 from MagneticField.
+"""
 
 import sys
 import subprocess
@@ -19,7 +27,7 @@ for lat in range(-90, 91, 5):
                               stdout=subprocess.PIPE,
                               stderr=subprocess.PIPE,
                               bufsize=0)
-        # magnetic variation varies, pin at 1 Jan 2020 
+        # magnetic variation varies, pin at 1 Jan 2020
         out, err = ge.communicate(b"2020-01-01 %d %d\n" % (lat, lon))
         # round to even cm
         parts = out.split()
