@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # clienthelpers.py - helper functions for xgps and test_maidenhead
@@ -7,8 +6,12 @@
 #
 # See gpsclient.c for code comments.
 #
+# This file is Copyright 2019 by the GPSD project
 # SPDX-License-Identifier: BSD-2-Clause
 """GPSd client helpers submodule."""
+
+from __future__ import absolute_import  # Ensure Python2 behaves like Python 3
+import gps
 import math
 import os
 
@@ -356,7 +359,7 @@ GEOID_DELTA = [
 #  This table is wmm2015.  Values obtained from MagneticField, part of
 #  geographiclib., by using devtools/get_mag_var_table.py
 #
-#  magvar[][] has the magnetic variation (declination), in hundreths of
+#  magvar[][] has the magnetic variation (declination), in hundredths of
 #  a degree, on a 5 degree by 5 * degree grid for the entire planet.
 #
 #  This table is duplicated in geoid.c.  Keep them in sync.
@@ -783,8 +786,11 @@ def deg_to_str(fmt, degrees):
 
 def gpsd_units():
     """Deduce a set of units from locale and environment."""
-    unit_lookup = {'imperial': imperial,
+    unit_lookup = {'i': imperial,
+                   'imperial': imperial,
+                   'm': metric,
                    'metric': metric,
+                   'n': nautical,
                    'nautical': nautical}
     if 'GPSD_UNITS' in os.environ:
         store = os.environ['GPSD_UNITS']
@@ -800,6 +806,36 @@ def gpsd_units():
     return unspecified
 
 
+class unit_adjustments(object):
+    """Encapsulate adjustments for unit systems."""
+
+    def __init__(self, units=None):
+        "Initialize class unit_adjustments"
+        self.altfactor = gps.METERS_TO_FEET
+        self.altunits = "ft"
+        self.name = "imperial"
+        self.speedfactor = gps.MPS_TO_MPH
+        self.speedunits = "mph"
+        if units is None:
+            units = gpsd_units()
+        if units in (unspecified, imperial, "imperial", "i"):
+            pass
+        elif units in (nautical, "nautical", "n"):
+            self.altfactor = gps.METERS_TO_FEET
+            self.altunits = "ft"
+            self.name = "nautical"
+            self.speedfactor = gps.MPS_TO_KNOTS
+            self.speedunits = "knots"
+        elif units in (metric, "metric", "m"):
+            self.altfactor = 1.0
+            self.altunits = "m"
+            self.name = "metric"
+            self.speedfactor = gps.MPS_TO_KPH
+            self.speedunits = "km/h"
+        else:
+            raise ValueError  # Should never happen
+
+
 # Arguments are in signed decimal latitude and longitude. For example,
 # the location of Montevideo (GF15vc) is: -34.91, -56.21166
 # plagarized from https://ham.stackexchange.com/questions/221
@@ -828,20 +864,34 @@ def maidenhead(dec_lat, dec_lon):
     adj_lat = dec_lat + 90.0
     adj_lon = dec_lon + 180.0
 
+    # divide into 18 zones (fields) each 20 degrees lon, 10 degrees lat
     grid_lat_sq = chr(int(adj_lat / 10) + 65)
     grid_lon_sq = chr(int(adj_lon / 20) + 65)
 
+    # divide into 10 zones (squares) each 2 degrees lon, 1 degrees lat
     grid_lat_field = str(int(adj_lat % 10))
     grid_lon_field = str(int((adj_lon / 2) % 10))
 
+    # remainder in minutes
     adj_lat_remainder = (adj_lat - int(adj_lat)) * 60
     adj_lon_remainder = ((adj_lon) - int(adj_lon / 2) * 2) * 60
 
+    # divide into 24 zones (subsquares) each 5 degrees lon, 2.5 degrees lat
     grid_lat_subsq = chr(97 + int(adj_lat_remainder / 2.5))
     grid_lon_subsq = chr(97 + int(adj_lon_remainder / 5))
 
-    return (grid_lon_sq + grid_lat_sq + grid_lon_field +
-            grid_lat_field + grid_lon_subsq + grid_lat_subsq)
+    # remainder in seconds
+    adj_lat_remainder = (adj_lat_remainder % 2.5) * 60
+    adj_lon_remainder = (adj_lon_remainder % 5.0) * 60
+
+    # divide into 10 zones (extended squares) each 30 secs lon, 15 secs lat
+    grid_lat_extsq = chr(48 + int(adj_lat_remainder / 15))
+    grid_lon_extsq = chr(48 + int(adj_lon_remainder / 30))
+
+    return (grid_lon_sq + grid_lat_sq +
+            grid_lon_field + grid_lat_field +
+            grid_lon_subsq + grid_lat_subsq +
+            grid_lon_extsq + grid_lat_extsq)
 
 
 def __bilinear(lat, lon, table):
@@ -902,3 +952,5 @@ def wgs84_separation(lat, lon):
     """Return MSL-WGS84 geodetic separation in meters.
 Given a lat/lon in degrees"""
     return __bilinear(lat, lon, GEOID_DELTA)
+
+# vim: set expandtab shiftwidth=4
