@@ -4,7 +4,7 @@
  * SPDX-License-Identifier: BSD-2-clause
  */
 
-#include "../gpsd_config.h"
+#include "../include/gpsd_config.h"
 
 #include <getopt.h>
 #include <stddef.h>
@@ -12,8 +12,8 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "../gpsd.h"
-#include "../gps_json.h"
+#include "../include/gpsd.h"
+#include "../include/gps_json.h"
 
 // Note: JSON_MINIMAL no longer exists
 
@@ -654,6 +654,7 @@ static void jsontest(int i)
 
     case 21:
         status = json_read_array(json_strInt, &json_array_Int, NULL);
+        assert_case(status);
         assert_integer("count", intcount, 3);
         assert_integer("intstore[0]", intstore[0], 23);
         assert_integer("intstore[1]", intstore[1], -17);
@@ -663,6 +664,7 @@ static void jsontest(int i)
 
     case 22:
         status = json_read_array(json_strBool, &json_array_Bool, NULL);
+        assert_case(status);
         assert_integer("count", boolcount, 3);
         assert_boolean("boolstore[0]", boolstore[0], true);
         assert_boolean("boolstore[1]", boolstore[1], false);
@@ -672,6 +674,7 @@ static void jsontest(int i)
 
     case 23:
         status = json_read_array(json_strReal, &json_array_Real, NULL);
+        assert_case(status);
         assert_integer("count", realcount, 3);
         assert_real("realstore[0]", realstore[0], 23.1);
         assert_real("realstore[1]", realstore[1], -17.2);
