@@ -7,19 +7,18 @@
  *
  */
 /* first so the #defs work */
-#include "../gpsd_config.h"
+#include "../include/gpsd_config.h"
 
 #include <math.h>
 #include <stdbool.h>
 #include <stdint.h>   /* required by C99, for int32_t */
 #include <stdio.h>
 #include <stdlib.h>
-#include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 
-#include "../compiler.h"       // for FALLTHROUGH
-#include "../gpsd.h"
+#include "../include/compiler.h"       // for FALLTHROUGH
+#include "../include/gpsd.h"
 
 #define TS_ZERO         {0,0}
 #define TS_ZERO_ONE     {0,1}
@@ -673,36 +672,49 @@ struct test_parse_uri_dest_t {
 // prefixed with gpsd:// in situ
 struct test_parse_uri_dest_t tests_parse_uri_dest[] = {
     {"localhost", "localhost", NULL, NULL},
+    {"localhost/", "localhost", NULL, NULL},
     {"localhost:", "localhost", NULL, NULL},
     {"localhost::", "localhost", NULL, NULL},
     {"localhost::/dev/ttyAMA0", "localhost", NULL, "/dev/ttyAMA0"},
     {"localhost:2947:/dev/ttyAMA0", "localhost", "2947", "/dev/ttyAMA0"},
+    {"localhost:2947", "localhost", "2947", NULL},
+    {"localhost:2947/", "localhost", "2947", NULL},
     {"localhost:gpsd", "localhost", "gpsd", NULL},
+    {"localhost:gpsd/", "localhost", "gpsd", NULL},
     {"gpsd.io", "gpsd.io", NULL, NULL},
+    {"gpsd.io/", "gpsd.io", NULL, NULL},
     {"gpsd.io:", "gpsd.io", NULL, NULL},
     {"gpsd.io::", "gpsd.io", NULL, NULL},
     {"gpsd.io::/dev/ttyAMA0", "gpsd.io", NULL, "/dev/ttyAMA0"},
     {"gpsd.io:2947:/dev/ttyAMA0", "gpsd.io", "2947", "/dev/ttyAMA0"},
+    {"gpsd.io:2947", "gpsd.io", "2947", NULL},
+    {"gpsd.io:2947/", "gpsd.io", "2947", NULL},
     {"gpsd.io:gpsd", "gpsd.io", "gpsd", NULL},
+    {"gpsd.io:gpsd/", "gpsd.io", "gpsd", NULL},
     {"127.0.0.1", "127.0.0.1", NULL, NULL},
+    {"127.0.0.1/", "127.0.0.1", NULL, NULL},
     {"127.0.0.1:", "127.0.0.1", NULL, NULL},
     {"127.0.0.1::", "127.0.0.1", NULL, NULL},
     {"127.0.0.1::/dev/ttyAMA0", "127.0.0.1", NULL, "/dev/ttyAMA0"},
     {"127.0.0.1:2947", "127.0.0.1", "2947", NULL},
+    {"127.0.0.1:2947/", "127.0.0.1", "2947", NULL},
     {"127.0.0.1:gpsd", "127.0.0.1", "gpsd", NULL},
+    {"127.0.0.1:gpsd/", "127.0.0.1", "gpsd", NULL},
     {"[fe80::1]", "fe80::1", NULL, NULL},
+    {"[fe80::1]/", "fe80::1", NULL, NULL},
     {"[fe80::1]:", "fe80::1", NULL, NULL},
     {"[fe80::1]::", "fe80::1", NULL, NULL},
     {"[fe80::1]::/dev/ttyAMA0", "fe80::1", NULL, "/dev/ttyAMA0"},
     {"[fe80::1]:2947", "fe80::1", "2947", NULL},
+    {"[fe80::1]:2947/", "fe80::1", "2947", NULL},
     {"[fe80::1]:gpsd", "fe80::1", "gpsd", NULL},
+    {"[fe80::1]:gpsd/", "fe80::1", "gpsd", NULL},
     {NULL, NULL, NULL, NULL},
 };
 
 static int test_parse_uri_dest(int verbose)
 {
     int fail_count = 0;
-    int result;
     char *host, *service, *device;
     struct test_parse_uri_dest_t *p = tests_parse_uri_dest;
     char uri[40];
@@ -710,6 +722,8 @@ static int test_parse_uri_dest(int verbose)
     printf("\n\nTest parse_uri_dest()\n");
 
     while(NULL != p->uri) {
+        int result;
+
         // parse_uri_dest() is destructive, so make a copy
         strlcpy(uri, p->uri, sizeof(uri));
 
